package cn.com.qmth.examcloud.service.examwork.api;

import cn.com.qmth.examcloud.common.uac.entity.AccessUser;
import cn.com.qmth.examcloud.service.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.service.examwork.entity.Exam;
import io.swagger.annotations.ApiOperation;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.common.util.ErrorMsg;
import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.service.examwork.assembler.ExamStudentAssembler;
import cn.com.qmth.examcloud.service.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.service.examwork.dto.ExamStudentDTO;
import cn.com.qmth.examcloud.service.examwork.entity.ExamStudent;
import cn.com.qmth.examcloud.service.examwork.service.ExamStudentService;
import cn.com.qmth.examcloud.service.examwork.util.ExportUtils;

/**
 * 考生服务API
 * Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}/exam_student")
public class ExamStudentApi {

    @Autowired
    ExamStudentRepo examStudentRepo;

    @Autowired
    ExamStudentService examStudentService;

    @Autowired
    ExamStudentAssembler examStudentAssembler;

    @Autowired
    ExamRepo examRepo;

    @ApiOperation(value = "查询考试学生带条件和分页", notes = "带条件带分页")
    @GetMapping("/all/{curPage}/{pageSize}")
    public ResponseEntity getAllExamStudent(HttpServletRequest request,
                                            @ModelAttribute ExamStudentDTO examStudent,
                                            @PathVariable Integer curPage,
                                            @PathVariable Integer pageSize) {
        AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
        if(accessUser != null){
            if(accessUser.getRootOrgId().longValue() == accessUser.getOrgId().longValue()){
                examStudent.setRootOrgId(accessUser.getRootOrgId());
            }else{
                examStudent.setOrgId(accessUser.getOrgId());
            }
        }else{
            return new ResponseEntity(HttpStatus.NOT_FOUND);
        }
        return new ResponseEntity(examStudentService.getAllExamStudent(examStudent,
                new PageRequest(curPage, pageSize)), HttpStatus.OK);
    }

    @ApiOperation(value = "查询所有考试学生带条件", notes = "带条件不分页")
    @GetMapping("/all")
    public ResponseEntity getAllExamStudent(HttpServletRequest request,
                                            @ModelAttribute ExamStudentDTO examStudent) {
        AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
        if(accessUser != null){
            if(accessUser.getRootOrgId().longValue() == accessUser.getOrgId().longValue()){
                examStudent.setRootOrgId(accessUser.getRootOrgId());
            }else{
                examStudent.setOrgId(accessUser.getOrgId());
            }
        }
        return new ResponseEntity(examStudentService.getAllExamStudent(examStudent), HttpStatus.OK);
    }

    @ApiOperation(value = "按ID查询考试学生", notes = "ID查询")
    @GetMapping("/{id}")
    public ResponseEntity<ExamStudent> getExamStudentById(@PathVariable Long id) {
        return new ResponseEntity<ExamStudent>(examStudentService.getExamStudentById(id), HttpStatus.OK);
    }

    @ApiOperation(value = "新增考试学生", notes = "新增")
    @PostMapping()
    public ResponseEntity addExamStudent(HttpServletRequest request,
                                         @RequestBody ExamStudent examStudent) {
        try {
            AccessUser accessUser = (AccessUser) request.getAttribute("accessUser");
            if (accessUser != null) {
                examStudent.setRootOrgId(accessUser.getRootOrgId());
            }
            Exam exam = examRepo.findOne(examStudent.getExam().getId());
            examStudent.setExam(exam);
            ExamStudent saveExamStu = examStudentService.saveExamStudent(examStudent);
            return new ResponseEntity(saveExamStu, HttpStatus.OK);
        } catch (Exception e){
            e.printStackTrace();
            return new ResponseEntity(new ErrorMsg(e.getMessage()),HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @ApiOperation(value = "更新考试学生", notes = "更新")
    @PutMapping()
    public ResponseEntity updateExamStudent(@RequestBody ExamStudent examStudent) {
        try {
            Exam exam = examRepo.findOne(examStudent.getExam().getId());
            examStudent.setExam(exam);
            ExamStudent saveExamStu = examStudentService.saveExamStudent(examStudent);
            return new ResponseEntity(saveExamStu, HttpStatus.OK);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResponseEntity(new ErrorMsg(e.getMessage()), HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @ApiOperation(value = "更新考试学生缺考状态", notes = "更新缺考")
    @PutMapping("/{id}")
    public ResponseEntity<ExamStudent> getExamStudentById(@PathVariable Long id, @RequestParam boolean finished) {
        ExamStudent examStudent = examStudentRepo.findOne(id);
        examStudent.setFinished(finished);
        return new ResponseEntity<ExamStudent>(examStudentRepo.save(examStudent), HttpStatus.OK);
    }

    @ApiOperation(value = "按ID删除考试学生", notes = "删除")
    @DeleteMapping("/{id}")
    public ResponseEntity deleteExamStudent(@PathVariable String id) {
        List<Long> examStuIds = Stream.of(id.split(",")).map(s->Long.parseLong(s.trim()))
                .collect(Collectors.toList());
        examStudentRepo.deleteInBatch(examStudentRepo.findByIdIn(examStuIds));
        return new ResponseEntity(HttpStatus.OK);
    }

    @ApiOperation(value = "导入考试学生", notes = "导入")
    @PostMapping("/import")
    public ResponseEntity importExamStudent(@RequestParam Long examId, @RequestParam MultipartFile file) {
        try {
            List<ExcelError> excelErrors = examStudentService.importExamStudent(examId, file.getInputStream());
            return new ResponseEntity(excelErrors, HttpStatus.OK);
        } catch (IOException e) {
            e.printStackTrace();
            return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

//    @ApiOperation(value="导入学生照片",notes = "导入")
//    @PostMapping("/exam_student/import_photo")
//    public ResponseEntity importPhoto(@RequestParam Long examId,@RequestParam MultipartFile file){
//    	try {
//    		List<ErrorMsg> errorMsgs = examStudentService.importPhoto(examId,file.getInputStream());
//    		return new ResponseEntity(errorMsgs,HttpStatus.OK);
//		} catch (IOException e) {
//			e.printStackTrace();
//			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
//		}
//    }

    @ApiOperation(value = "照片检验", notes = "检验")
    @PostMapping("/photo_check")
    public ResponseEntity photoCheck(@RequestParam Long examId) {
        try {
            List<ErrorMsg> errorMsgs = examStudentService.photoCheck(examId);
            return new ResponseEntity(errorMsgs, HttpStatus.OK);
        } catch (Exception e) {
            e.printStackTrace();
            return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
        }
    }

    @ApiOperation(value = "导出考试学生带条件", notes = "导出")
    @GetMapping("/export")
    public void exportExamStudent(@ModelAttribute ExamStudentDTO examCriteria, HttpServletResponse response) {
        List<ExamStudentDTO> list = new ArrayList<ExamStudentDTO>();
        examStudentService.getAllExamStudent(examCriteria).forEach(c -> {
            list.add(examStudentAssembler.toDTO(c));
        });
        ExportUtils.exportEXCEL("课程列表", ExamStudentDTO.class, list, response);
    }
}
